/*
 * @(#)AList 31 October 2006
 *
 * @author Viera K. Proulx
 */
/**
 * An interface to represent a recursively built list of elements
 * of the type <CODE>T</CODE> that also implements the 
 * <CODE>{@link ISame ISame}</CODE> interface.
 * It extends the <CODE>{@link Traversal Traversal}</CODE> interface.
 */
public interface AList<T extends ISame> extends Traversal<T>{
  
  /**
   * Does this list contain the given object?
   * 
   * @param obj the given object.
   */ 
  public boolean contains(T obj);
  
  /**
   * Count the items in this list.
   */ 
  public int count();
  
  /** 
   * Produce a traversal for the collection with the first element removed.
   * Throw an exception if the first element cannot be removed.
   * 
   * @return new traversal for a collection without the first element.
   * @throws IllegalUseOfTraversalException 
   * if the first elemment cannot be removed.
   * 
   */
  public Traversal<T> getRest() throws IllegalUseOfTraversalException;  
  
  /** 
   * Provide the first element of this collection.
   * Throw an exception if there is no first element.
   * 
   * @return the first element in the collection.
   * @throws IllegalUseOfTraversalException 
   * if there is no first element.
   */
  public T getFirst() throws IllegalUseOfTraversalException; 
  
  /** 
   * Test whether there are any items in the collection.
   * 
   * @return true if the collection has no elements.
   */
  public boolean isEmpty(); 
}
