/*
 * @(#)ConsList 31 October 2006
 *
 * @author Viera K. Proulx
 */
/**
 * A class to represent a nonempty list of elements
 * of the type <CODE>T</CODE> that also implements the 
 * <CODE>{@link ISame ISame}</CODE> interface.
 * It extends the <CODE>{@link Traversal Traversal}</CODE> interface.
 */
public class ConsList<T extends ISame> implements AList<T>{
  
  /**
   * the first element in this list.
   */
  public T first;
  /**
   * the rest of this list.
   */
  public AList<T> rest;
  
  /**
   * The full constructor.
   * @param first the first element of this list.
   * @param rest the rest of this list.
   */
  public ConsList(T first, AList<T> rest){
    this.first = first;
    this.rest = rest;
  }
  
  /**
   * Does this list contain the given object?
   * 
   * @param obj the given object.
   */ 
  public boolean contains(T obj){ 
    if (obj.same(this.first))
      return true;
    else
      return this.rest.contains(obj);
  }
  
  /**
   * Count the items in this list.
   */ 
  public int count(){ 
    return 1 + this.rest.count(); 
  }
  
  /** 
   * Produce a traversal for the collection with the first element removed.
   * 
   * @return new traversal for a collection without the first element.
   */
  public Traversal<T> getRest() throws IllegalUseOfTraversalException{
    return this.rest;
  }
  
  /** 
   * Provide the first element of this collection.
   * 
   * @return the first element in the collection.
   */
  public T getFirst() throws IllegalUseOfTraversalException{
    return this.first;
  } 
  
  /** 
   * Test whether there are any items in the collection.
   * 
   * @return true if the collection has no elements.
   */
  public boolean isEmpty(){
    return false;
  }  
  
  /**
   * Represent this instance as a <CODE>String</CODE>.
   */ 
  public String toString(){
    return "new ConsList(\n first = " + this.first.toString() + 
    "\n this.rest = " + this.rest.toString() + ")";
  }
}
