/*
 * @(#)Examples 31 October 2006
 *
 * @author Viera K. Proulx
 */
import java.awt.*;

/**
 * The class that contains all test cases
 */
public class Examples {
  /**
   * The test harness.
   */ 
  TestHarness th = new TestHarness();
  
  /*-----------------------------------------------------------------------*/  
  /**
   * Balloon data.
   */ 
  Balloon b1 = new Balloon(new CartPt(20, 40), 30, new Color(255, 0, 0));
  /**
   * Balloon data.
   */ 
  Balloon b2 = new Balloon(new CartPt(40, 30), 20, new Color(0, 255, 0));
  /**
   * Balloon data.
   */ 
  Balloon b3 = new Balloon(new CartPt(40, 20), 10, new Color(0, 0, 255));
  /**
   * Balloon data.
   */ 
  Balloon b4 = new Balloon(new CartPt(20, 20), 25, new Color(255, 0, 255));
  /*-----------------------------------------------------------------------*/
  /**
   * ImageFile data.
   */ 
  ImageFile pic1 = new ImageFile("party", 300, 200);
  /**
   * ImageFile data.
   */ 
  ImageFile pic2 = new ImageFile("river", 100, 200);
  /**
   * ImageFile data.
   */ 
  ImageFile pic3 = new ImageFile("woods", 200, 200);
  
  /*-----------------------------------------------------------------------*/ 
  /**
   * TopThree data with balloon objects.
   */
  TopThree<Balloon> ttr = 
    new TopThree<Balloon>(this.b1, this.b2, this.b3, new BalloonByRadius());
  /**
   * TopThree data with balloon objects.
   */  TopThree<Balloon> tth = 
     new TopThree<Balloon>(this.b3, this.b2, this.b1, new BalloonByHeight());
   /**
    * TopThree data with image file objects.
    */ TopThree<ImageFile> ttPicName = 
      new TopThree<ImageFile>(this.pic1, this.pic2, this.pic3, 
          new ImageFileByName());
    /**
     * TopThree data with image file objects.
     */  
    TopThree<ImageFile> ttPicSize = 
      new TopThree<ImageFile>(this.pic2, this.pic3, this.pic1, 
          new ImageFileBySize());
    
    /*-----------------------------------------------------------------------*/ 
    /**
     * A list of balloons.
     */
    AList<Balloon> mtballoons= new MTList<Balloon>();
    
    /**
     * A list of balloons.
     */
    AList<Balloon> balloons= 
      new ConsList<Balloon>(b1,
          new ConsList<Balloon>(b2,
              new ConsList<Balloon>(b3,
                  new MTList<Balloon>())));
    
    /*-----------------------------------------------------------------------*/ 
    /**
     * A list of image files.
     */
    AList<ImageFile> mtpictures = new MTList<ImageFile>();
    
    /**
     * A list of image files.
     */    
    AList<ImageFile> pictures= 
      new ConsList<ImageFile>(pic1,
          new ConsList<ImageFile>(pic2,
              new MTList<ImageFile>()));
    
    /*-----------------------------------------------------------------------*/ 
    /**
     * Display all data used for the tests.
     */
    public void showData(){
      System.out.println("Balloon b1 = " + this.b1.toString());
      System.out.println("Balloon b2 = " + this.b2.toString());
      System.out.println("Balloon b3 = " + this.b3.toString());
      System.out.println("Balloon b4 = " + this.b4.toString());
      System.out.println();
      
      System.out.println("ImageFile pic1 = " + this.pic1.toString());
      System.out.println("ImageFile pic2 = " + this.pic2.toString());
      System.out.println("ImageFile pic3 = " + this.pic3.toString());
      System.out.println();
      
      System.out.println("AList balloons = " + this.balloons.toString());
      System.out.println();
      System.out.println("AList pictures = " + this.pictures.toString());
      System.out.println();  
      
      System.out.println("TopThree ttr = " + this.ttr.toString());
      System.out.println("TopThree tth = " + this.tth.toString());
      System.out.println("TopThree ttPicName = " + this.ttPicName.toString());
      System.out.println("TopThree ttPicSize = " + this.ttPicSize.toString());
    }
    
    /*-----------------------------------------------------------------------*/ 
    /**
     * Reset all data used for the tests.
     */ 
    public void resetData(){
      
      this.b1 = new Balloon(new CartPt(20, 40), 30, new Color(255, 0, 0));
      this.b2 = new Balloon(new CartPt(40, 30), 20, new Color(0, 255, 0));
      this.b3 = new Balloon(new CartPt(40, 20), 10, new Color(0, 0, 255));
      this.b4 = new Balloon(new CartPt(20, 20), 25, new Color(255, 0, 255));
      
      this.pic1 = new ImageFile("party", 300, 200);
      this.pic2 = new ImageFile("river", 100, 200);
      this.pic3 = new ImageFile("woods", 200, 200);
      
      this.balloons = 
        new ConsList<Balloon>(b1,
            new ConsList<Balloon>(b2,
                new ConsList<Balloon>(b3,
                    new MTList<Balloon>())));
      
      this.pictures = 
        new ConsList<ImageFile>(pic1,
            new ConsList<ImageFile>(pic2,
                new MTList<ImageFile>())); 
      
      this.ttr = 
        new TopThree<Balloon>(this.b1, this.b2, this.b3, this.ttr.comp);
      this.tth = 
        new TopThree<Balloon>(this.b3, this.b2, this.b1, this.tth.comp);  
      this.ttPicName = 
        new TopThree<ImageFile>(this.pic1, this.pic2, this.pic3, 
            new ImageFileByName());
      this.ttPicSize = 
        new TopThree<ImageFile>(this.pic2, this.pic3, this.pic1, 
            new ImageFileBySize());
    }
    
    /*-----------------------------------------------------------------------*/ 
    /**
     * Test the method inflateBalloon in the class Balloon.
     */ 
    public void testInflateBalloon(){
      this.resetData();
      this.b4.inflateBalloon(5);
      th.test("inflateBalloon 1: ", this.b4.radius, 30);    
    }
    
    /*-----------------------------------------------------------------------*/ 
    /**
     * Test the method change in the class Balloon.
     */ 
    public void testChangeBalloon(){
      this.resetData();
      this.b4.change(5);
      th.test("change balloon 1: ", this.b4.radius, 30);    
    }
    
    /**
     * Test the method floatBalloon in the class Balloon.
     */ 
    public void testFloatBalloon(){
      this.resetData();
      this.b4.floatBalloon(5);
      th.test("floatBalloon 1: ", this.b4.loc.y, 15);    
    }
    
    /*-----------------------------------------------------------------------*/ 
    /**
     * Test the constructor in the class TopThree.
     * Verify that data is ordered as desired.
     */ 
    public void testConstructor(){
      this.resetData();
      th.test("test constructors 1: ", this.tth.all.get(0), this.b3);    
      th.test("test constructors 2: ", this.tth.all.get(1), this.b2);    
      th.test("test constructors 3: ", this.tth.all.get(2), this.b1);    
      
      th.test("test constructors 4: ", this.ttPicSize.all.get(0), this.pic1);    
      th.test("test constructors 5: ", this.ttPicSize.all.get(1), this.pic3);    
      th.test("test constructors 6: ", this.ttPicSize.all.get(2), this.pic2);    
    }
    
    /*-----------------------------------------------------------------------*/ 
    /**
     * Test the method change in the class TopThree.
     */ 
    public void testChange(){
      this.resetData();
      this.ttr.change(-10, 5, 20);
      th.test("change balloons 1: ", this.ttr.all.get(0), this.b3);    
      th.test("change balloons 2: ", this.ttr.all.get(1), this.b2);    
      th.test("change balloons 3: ", this.ttr.all.get(2), this.b1);    
    }
    
    /*-----------------------------------------------------------------------*/ 
    /**
     * Test the method change in the class ImageFile.
     */ 
    public void testChangeImageFile(){
      this.resetData();
      this.pic3.change(5);
      th.test("change image file width: ", this.pic3.width, 195);    
      th.test("change image file height: ", this.pic3.height, 195);    
    }
    
    /*-----------------------------------------------------------------------*/ 
    /**
     * Test the method count in the class AList.
     */ 
    public void testCount(){
      this.resetData();
      th.test("count balloons 1: ", this.balloons.count(), 3);    
      th.test("count balloons 3: ", this.mtballoons.count(), 0);    
      
      th.test("count pictures 1: ", this.pictures.count(), 2);    
      th.test("count pictures 2: ", this.mtpictures.count(), 0);    
    }
    
    /*-----------------------------------------------------------------------*/ 
    /**
     * Test the method contains in the class AList.
     */ 
    public void testContains(){
      this.resetData();
      th.test("contains balloons 1: ", this.balloons.contains(this.b2), true);    
      th.test("contains balloons 2: ", this.balloons.contains(this.b4), false);    
      th.test("contains balloons 3: ", this.mtballoons.contains(this.b2), false);    
      
      th.test("contains pictures 1: ", this.pictures.contains(this.pic2), true);    
      th.test("contains pictures 2: ", this.pictures.contains(this.pic3), false);    
      th.test("contains pictures 3: ", this.mtpictures.contains(this.pic2), false);    
    }
    
    /*-----------------------------------------------------------------------*/ 
    /**
     * Test the method change in the class TopThree.
     */ 
    public void testChangePicSize(){
      this.resetData();
      this.ttPicSize.change(100, -200, -50);
      th.test("change size 1: ", this.ttPicSize.all.get(0), this.pic3);    
      th.test("change size 2: ", this.ttPicSize.all.get(1), this.pic2);    
      th.test("change size 3: ", this.ttPicSize.all.get(2), this.pic1);    
    }
    
    /*-----------------------------------------------------------------------*/ 
    /**
     * The main method to run all tests.
     */
    public static void main(String[] argv){
      Examples e = new Examples();
      
      e.showData();
      
      e.testInflateBalloon();
      e.testFloatBalloon();
      e.testChangeBalloon();
      e.testChangeImageFile();
      e.testConstructor();
      e.testChange();
      e.testChangePicSize();
      
      e.th.testReport();
      e.th.fullTestReport();
    }
}